@@ -1,5 +1,5 @@
-stan_llh <- function(family, effects = list(), autocor = cor_arma(),
-                     trunc_bounds = NULL) {
+stan_llh <- function(family, effects = list(), data = NULL, 
+                     autocor = cor_arma()) {
   # Likelihood in Stan language
   # Args:
   #   family: the model family
@@ -17,11 +17,12 @@ stan_llh <- function(family, effects = list(), autocor = cor_arma(),
   
   has_se <- is.formula(effects$se)
   has_weights <- is.formula(effects$weights)
-  has_cens <- is.formula(effects$cens)
+  has_cens <- has_cens(effects$cens, data = data)
   has_disp <- is.formula(effects$disp)
   has_trials <- is.formula(effects$trials)
-  has_cse <- is.formula(effects$cse)
-  has_trunc <- any(trunc_bounds$lb > -Inf) || any(trunc_bounds$ub < Inf)
+  has_cse <- has_cse(effects)
+  bounds <- get_bounds(effects$trunc, data = data)
+  has_trunc <- any(bounds$lb > -Inf) || any(bounds$ub < Inf)
   ll_adj <- has_cens || has_weights || has_trunc
 
   if (is_mv) {
@@ -61,13 +62,14 @@ stan_llh <- function(family, effects = list(), autocor = cor_arma(),
   .logit <- ifelse(any(c("zi", "hu") %in% auxpars), "_logit", "")
   reqn_trials <- has_trials && (ll_adj || is_zero_inflated)
   trials <- ifelse(reqn_trials, "trials[n]", "trials")
-  ordinal_args <- paste("eta[n],", if (has_cse) "etap[n],", "temp_Intercept")
-  
-  # use inverse link in likelihood statement only 
-  # if it does not prevent vectorization 
+
   simplify <- !has_trunc && !has_cens && stan_has_built_in_fun(family, link)
   eta <- paste0(ifelse(is_mv, "Eta", "eta"), n)
-  
+  ordinal_args <- paste0("eta[n], ", if (has_cse) "etacs[n], ", 
+                         "temp_Intercept")
+  inv_gauss_fun <- paste0("inv_gaussian", if (!reqn) "_vector")
+  inv_gauss_args <- paste0(eta, ", shape, ", if (!reqn) "sum_", 
+                           "log_Y", n, ", sqrt_Y", n)
   if (simplify) { 
     llh_pre <- switch(family,
       poisson = c("poisson_log", eta), 
@@ -98,8 +100,7 @@ stan_llh <- function(family, effects = list(), autocor = cor_arma(),
       gamma = c("gamma", paste0(shape, ", ", eta)), 
       exponential = c("exponential", eta),
       weibull = c("weibull", paste0(shape,", ", eta)), 
-      inverse.gaussian = c(paste0("inv_gaussian", if (!reqn) "_vector"), 
-                           paste0(eta, ", shape, log_Y", n, ", sqrt_Y", n)),
+      inverse.gaussian = c(inv_gauss_fun, inv_gauss_args),
       beta = c("beta", paste0(eta, " * ", phi, ", (1 - ", eta, ") * ", phi)),
       von_mises = c(paste0("von_mises_", ifelse(reqn, "real", "vector")), 
                            paste0(eta, ", ", kappa)),
@@ -125,46 +126,74 @@ stan_llh <- function(family, effects = list(), autocor = cor_arma(),
   }
   
   # write likelihood code
+  interval <- isTRUE(attr(has_cens, "interval"))
   type <- c("cens", "weights")[match(TRUE, c(has_cens, has_weights))]
   if (is.na(type)) type <- "general"
-  # prepare for possible truncation
-  code_trunc <- ""
-  if (has_trunc) {
-    if (type %in% c("cens", "weights")) {
-      stop("truncation is not yet possible in censored ", 
-           "or weighted models", call. = FALSE)
-    } else {
-      lb <- ifelse(any(trunc_bounds$lb > -Inf), "lb[n]", "")
-      ub <- ifelse(any(trunc_bounds$ub < Inf), "ub[n]", "")
-      code_trunc <- paste0(" T[", lb, ", ", ub, "]")
-    }
-  }
-  weights <- ifelse(has_weights, "weights[n] * ", "")
-  .lpdf <- ifelse(use_int(family), "_lpmf", "_lpdf")
   llh <- switch(type, 
-    cens = paste0("  // special treatment of censored data \n",
-      "      if (cens[n] == 0) ", 
-      ifelse(!has_weights, 
-        paste0("Y[n] ~ ", llh_pre[1], "(", llh_pre[2],"); \n"),
-        paste0("target += ", weights, llh_pre[1], .lpdf, 
-               "(Y[n] | ", llh_pre[2],"); \n")),
-      "      else { \n",         
-      "        if (cens[n] == 1) target += ",
-      weights, llh_pre[1], "_lccdf(Y[n] | ", llh_pre[2],"); \n",
-      "        else target += ", 
-      weights, llh_pre[1], "_lcdf(Y[n] | ", llh_pre[2],"); \n",
-      "      } \n"),
-    weights = paste0("  lp_pre[n] = ", llh_pre[1], .lpdf, "(Y[n] | ",
-                     llh_pre[2],"); \n"),
-    general = paste0("  Y", n, " ~ ", llh_pre[1], "(", llh_pre[2], ")", 
-                     code_trunc, "; \n")) 
-  # loop over likelihood if it cannot be vectorized
+    cens = stan_llh_cens(llh_pre, family, interval, has_weights),
+    weights = stan_llh_weights(llh_pre, family),
+    general = stan_llh_general(llh_pre, reqn, bounds)) 
   if (reqn) {
+    # loop over likelihood if it cannot be vectorized
     llh <- paste0("  for (n in 1:N) { \n    ", llh, "    } \n")
   }
   llh
 }
 
+stan_llh_general <- function(llh_pre, reqn = FALSE, bounds = NULL) {
+  # default likelihood in Stan language
+  # Args:
+  #   reqn: does Y require the index 'n'?
+  #   bounds: a list containing elements lb and ub
+  stopifnot(length(llh_pre) == 2L)
+  if (any(bounds$lb > -Inf) || any(bounds$ub < Inf)) {
+    # prepare possible truncation
+    lb <- ifelse(any(bounds$lb > -Inf), "lb[n]", "")
+    ub <- ifelse(any(bounds$ub < Inf), "ub[n]", "")
+    trunc <- paste0(" T[", lb, ", ", ub, "]")
+  } else {
+    trunc <- ""
+  }
+  paste0("  Y", ifelse(reqn, "[n]", ""), " ~ ", llh_pre[1], 
+         "(", llh_pre[2], ")", trunc, "; \n")
+}
+
+stan_llh_cens <- function(llh_pre, family = gaussian(), 
+                          interval = FALSE, weights = FALSE) {
+  # censored likelihood in Stan language
+  # Args: 
+  #   interval: are there interval censored responses present?
+  #   weights: is the model additionally weighted?
+  stopifnot(length(llh_pre) == 2L)
+  s <- collapse(rep(" ", 6))
+  tp <- "  target += "
+  lpdf <- ifelse(use_int(family), "lpmf", "lpdf")
+  w <- ifelse(weights, "weights[n] * ", "")
+  paste0("  // special treatment of censored data \n",
+    s, "if (cens[n] == 0) \n", 
+    s, tp, w, llh_pre[1], "_", lpdf, "(Y[n] | ", llh_pre[2], "); \n",
+    s, "else if (cens[n] == 1) \n",         
+    s, tp, w, llh_pre[1], "_lccdf(Y[n] | ", llh_pre[2], "); \n",
+    s, "else if (cens[n] == -1) \n",
+    s, tp, w, llh_pre[1], "_lcdf(Y[n] | ", llh_pre[2], "); \n",
+    if (interval) {
+      paste0(
+        s, "else if (cens[n] == 2) \n",
+        s, tp, w, "log_diff_exp(", 
+          llh_pre[1], "_lcdf(rcens[n] | ", llh_pre[2], "), \n",
+        collapse(rep(" ", 31)),
+          llh_pre[1], "_lcdf(Y[n] | ", llh_pre[2], ")); \n")
+    })
+}
+
+stan_llh_weights <- function(llh_pre, family = gaussian()) {
+  # weighted likelihood in Stan language
+  stopifnot(length(llh_pre) == 2L)
+  lpdf <- ifelse(use_int(family), "lpmf", "lpdf")
+  paste0("  lp_pre[n] = ", llh_pre[1], "_", lpdf, 
+         "(Y[n] | ", llh_pre[2],"); \n")
+}
+
 stan_autocor <- function(autocor, effects = list(), family = gaussian(),
                          prior = prior_frame()) {
   # Stan code related to autocorrelation structures
@@ -383,14 +412,12 @@ stan_mv <- function(family, response, prior = prior_frame()) {
 stan_ordinal <- function(family, prior = prior_frame(), 
                          cse = FALSE, threshold = "flexible") {
   # Ordinal effects in Stan
-  #
   # Args:
   #   family: the model family
   #   prior: a data.frame containing user defined priors 
   #          as returned by check_prior
   #   cse: logical; are there category specific effects?
   #   threshold: either "flexible" or "equidistant" 
-  #
   # Returns:
   #   A vector of strings containing the ordinal effects in stan language
   stopifnot(is(family, "family"))
@@ -400,8 +427,8 @@ stan_ordinal <- function(family, prior = prior_frame(),
     out$data <- "  int ncat;  // number of categories \n"
     th <- function(k, fam = family) {
       # helper function generating stan code inside ilink(.)
-      sign <- ifelse(fam %in% c("cumulative", "sratio")," - ", " + ")
-      ptl <- ifelse(cse, paste0(sign, "etap[k]"), "") 
+      sign <- ifelse(fam %in% c("cumulative", "sratio"), " - ", " + ")
+      ptl <- ifelse(cse, paste0(sign, "etacs[k]"), "") 
       if (sign == " - ") {
         out <- paste0("thres[",k,"]", ptl, " - eta")
       } else {
@@ -433,13 +460,13 @@ stan_ordinal <- function(family, prior = prior_frame(),
     
     # generate Stan code specific for each ordinal model
     if (!(family == "cumulative" && ilink == "inv_logit")) {
-      cse_arg <- ifelse(!cse, "", "row_vector etap, ")
+      cse_arg <- ifelse(!cse, "", "row_vector etacs, ")
       out$fun <- paste0(
         "  /* ", family, " log-PDF for a single response \n",
         "   * Args: \n",
         "   *   y: response category \n",
         "   *   eta: linear predictor \n",
-        "   *   etap: optional linear predictor for category specific effects \n",
+        "   *   etacs: optional predictor for category specific effects \n",
         "   *   thres: ordinal thresholds \n",
         "   * Returns: \n", 
         "   *   a scalar to be added to the log posterior \n",
@@ -507,7 +534,7 @@ stan_categorical <- function(family) {
   stopifnot(is(family, "family"))
   out <- list()
   if (is.categorical(family)) {
-    out$data <- "  int<lower=2> ncat;  // number of categories' \n" 
+    out$data <- "  int<lower=2> ncat;  // number of categories \n" 
     out$tdataD <- "  vector[1] zero; \n"
     out$tdataC <- "  zero[1] = 0; \n"
   }
@@ -550,7 +577,7 @@ stan_forked <- function(family) {
   out
 }
 
-stan_inv_gaussian <- function(family, ll_adj = FALSE) {
+stan_inv_gaussian <- function(family) {
   # stan code for inverse gaussian models
   # Args:
   #   family: the model family
@@ -562,14 +589,8 @@ stan_inv_gaussian <- function(family, ll_adj = FALSE) {
   out <- list()
   if (family$family == "inverse.gaussian") {
     out$fun <- "  #include 'fun_inv_gaussian.stan' \n"
-    out$data <- paste0(
-      "  // quantities for the inverse gaussian distribution \n",
-      "  vector[N] sqrt_Y;  // sqrt(Y) \n")
-    if (ll_adj) {
-      out$data <- paste0(out$data, "  vector[N] log_Y;  // log(Y) \n")
-    } else {
-      out$data <- paste0(out$data, "  real log_Y;  // sum(log(Y)) \n")
-    }
+    out$tdataD <- "  #include 'tdataD_inv_gaussian.stan' \n"
+    out$tdataC <- "  #include 'tdataC_inv_gaussian.stan' \n"
   }
   out
 }
@@ -585,6 +606,20 @@ stan_von_mises <- function(family, ...) {
   out
 }
 
+stan_cens <- function(cens, family = gaussian()) {
+  out <- list()
+  if (cens) {
+    stopifnot(is(family, "family"))
+    out$data <- paste0(
+      "  int<lower=-1,upper=2> cens[N];  // indicates censoring \n",
+      if (isTRUE(attr(cens, "interval"))) {
+        paste0(ifelse(use_int(family), " int rcens[N];", "  vector[N] rcens;"),
+               "  // right censor points for interval censoring \n")
+      })
+  }
+  out
+}
+
 stan_disp <- function(effects, family = gaussian()) {
   # stan code for models with addition argument 'disp'
   # Args:
@@ -596,10 +631,13 @@ stan_disp <- function(effects, family = gaussian()) {
     par <- if (has_sigma(family)) "sigma"
            else if (has_shape(family)) "shape"
            else stop("invalid family for addition argument 'disp'")
-    times <- if(is.null(effects[[par]])) " * " else " .* "
+    if (!is.null(effects[[par]])) {
+      stop("Specifying 'disp' is not allowed when predicting '", 
+           par, "'", call. = FALSE)
+    }
     out$data <- "  vector<lower=0>[N] disp;  // dispersion factors \n"
     out$modelD <- paste0("  vector[N] disp_", par, "; \n")
-    out$modelC1 <- paste0("  disp_", par, " = ", par, times, "disp; \n")
+    out$modelC1 <- paste0("  disp_", par, " = ", par, " * disp; \n")
   }
   out
 }
@@ -900,9 +938,9 @@ stan_needs_kronecker <- function(ranef, names_cov_ranef) {
   ids <- unique(ranef$id)
   out <- FALSE
   for (id in ids) {
-    id_ranef <- subset(ranef, id == id)
-    out <- out || nrow(id_ranef) > 1L && id_ranef$cor[1] &&
-      id_ranef$group[1] %in% names_cov_ranef
+    r <- ranef[ranef$id == id, ]
+    out <- out || nrow(r) > 1L && r$cor[1] && 
+             r$group[1] %in% names_cov_ranef
   }
   out
 }
